@@ -1,4 +1,4 @@
-# Copyright 2018 Hewlett Packard Enterprise Development LP
+# Copyright 2018-2022 Hewlett Packard Enterprise Development LP
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -16,21 +16,25 @@ __author__ = 'HPE'
 
 from base64 import b64decode
 import json
+import os
 import re
+import subprocess
+import tempfile
 
 from OpenSSL.crypto import FILETYPE_ASN1
 from OpenSSL.crypto import load_certificate
 from six.moves.urllib import parse
 import sushy
-from sushy.resources.system import mappings as sushy_map
 from sushy import utils
 
 from proliantutils import exception
+from proliantutils.ilo import common
 from proliantutils.ilo import constants as ilo_cons
 from proliantutils.ilo import firmware_controller
 from proliantutils.ilo import operations
 from proliantutils import log
 from proliantutils.redfish import main
+from proliantutils.redfish.resources import gpu_common
 from proliantutils.redfish.resources.manager import constants as mgr_cons
 from proliantutils.redfish.resources.system import constants as sys_cons
 from proliantutils.redfish.resources.system.storage \
@@ -101,6 +105,7 @@ GET_POST_STATE_MAP = {
 # collection, as we are dealing with iLO's here.
 PROLIANT_MANAGER_ID = '1'
 PROLIANT_SYSTEM_ID = '1'
+PROLIANT_CHASSIS_ID = '1'
 
 BOOT_OPTION_MAP = {'BOOT_ONCE': True,
                    'BOOT_ALWAYS': False,
@@ -122,6 +127,15 @@ _CERTIFICATE_PATTERN = (
 
 LOG = log.get_logger(__name__)
 
+# Copied from ironic/drivers/modules/redfish/inspect.py
+CPU_ARCH_MAP = {
+    sushy.PROCESSOR_ARCH_x86: 'x86_64',
+    sushy.PROCESSOR_ARCH_IA_64: 'ia64',
+    sushy.PROCESSOR_ARCH_ARM: 'arm',
+    sushy.PROCESSOR_ARCH_MIPS: 'mips',
+    sushy.PROCESSOR_ARCH_OEM: 'oem'
+}
+
 
 class RedfishOperations(operations.IloOperations):
     """Operations supported on redfish based hardware.
@@ -214,6 +228,24 @@ class RedfishOperations(operations.IloOperations):
             LOG.debug(msg)
             raise exception.IloError(msg)
 
+    def _get_sushy_chassis(self, chassis_id):
+        """Get the sushy chassis for chassis_id
+
+        :param chassis_id: The identity of the Chassis resource
+        :returns: the Sushy Chassis instance
+        :raises: IloError
+        """
+        chassis_url = parse.urljoin(self._sushy.get_chassis_collection_path(),
+                                    chassis_id)
+        try:
+            return self._sushy.get_chassis(chassis_url)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish Chassis "%(chassis)s" was not found. '
+                          'Error %(error)s') %
+                   {'chassis': chassis_id, 'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
     def get_product_name(self):
         """Gets the product name of the server.
 
@@ -424,6 +456,8 @@ class RedfishOperations(operations.IloOperations):
         :raises: IloError, on an error from iLO.
         :raises: IloInvalidInputError, if the device is not valid.
         """
+        # Validate url
+        common_utils.validate_href(url)
         self._validate_virtual_media(device)
         manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
         try:
@@ -660,7 +694,391 @@ class RedfishOperations(operations.IloOperations):
             LOG.debug(msg)
             raise exception.IloError(msg)
 
-    def _get_security_dashboard_values(self):
+    def _update_security_parameter(self, sec_param, ignore=False):
+        """Sets the ignore flag for the security parameter.
+
+        :param sec_param: Name of the security parameter.
+        :param ignore : True when security parameter needs to be ignored.
+               If passed False, security param will not be ignored.
+               If nothing passed default will be False.
+        """
+        sushy_manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            security_params = (
+                sushy_manager.securityservice.securityparamscollectionuri)
+            param_members = security_params.get_members()
+            for param in param_members:
+                if sec_param in param.name:
+                    param.update_security_param_ignore_status(ignore)
+                    break
+            else:
+                msg = (self._('Specified parameter "%(param)s" is not '
+                              'a Security Dashboard Parameter.') %
+                       {'param': sec_param})
+                raise exception.IloInvalidInputError(msg)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._("The Redfish controller is unable to update "
+                          "resource or its member. Error "
+                          "%(error)s)") % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_password_complexity(self, enable=True, ignore=False):
+        """Update the Password_Complexity security param.
+
+        :param enable: A boolean param, True when Password_Complexity needs
+               to be enabled. If passed False, Password_Complexity security
+               param will be disabled. If nothing passed default will be True.
+        :param ignore : A boolean param, True when Password_Complexity needs
+               to be ignored. If passed False, Password_Complexity security
+               param will not be ignored. If nothing passed default will be
+               False.
+        :raises: IloError, on an error from iLO.
+        """
+        acc_service = self._sushy.get_account_service()
+        try:
+            self._update_security_parameter(sec_param="Password Complexity",
+                                            ignore=ignore)
+            acc_service.update_enforce_passwd_complexity(enable)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard parameter '
+                          '``Password_Complexity``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_require_login_for_ilo_rbsu(self, enable=True, ignore=False):
+        """Update the RequiredLoginForiLORBSU security param.
+
+        :param enable: A boolean param, True when RequiredLoginForiLORBSU
+               needs to be enabled. If passed False, RequiredLoginForiLORBSU
+               security param will be disabled. If nothing passed default
+               will be True.
+        :param ignore : A boolean param, True when RequiredLoginForiLORBSU
+               needs to be ignored. If passed False, RequiredLoginForiLORBSU
+               security param will not be ignored. If nothing passed default
+               will be False.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            self._update_security_parameter(sec_param="Require Login",
+                                            ignore=ignore)
+            sushy_manager.update_login_for_ilo_rbsu(enable)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard parameter '
+                          '``RequiredLoginForiLORBSU``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_require_host_authentication(self, enable=True, ignore=False):
+        """Update the RequireHostAuthentication security param.
+
+        :param enable: A boolean param, True when RequireHostAuthentication
+               needs to be enabled. If passed False, RequireHostAuthentication
+               security param will be disabled. If nothing passed
+               default will be True.
+        :param ignore : A boolean param, True when RequireHostAuthentication
+               needs to be ignored. If passed False, RequireHostAuthentication
+               security param will not be ignored. If nothing passed
+               default will be False.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            self._update_security_parameter(sec_param="Host Authentication",
+                                            ignore=ignore)
+            sushy_manager.update_host_authentication(enable)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard paramater '
+                          '``RequireHostAuthentication``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_minimum_password_length(self, passwd_length=None, ignore=False):
+        """Update the MinPasswordLength security param.
+
+        :param passwd_length: Minimum lenght of password used. If nothing
+               passed default will be None.
+        :param ignore : A boolean param, True when MinPasswordLength needs to
+               be ignored. If passed False, MinPasswordLength security param
+               will not be ignored. If nothing passed default will be False.
+        """
+        acc_service = self._sushy.get_account_service()
+        try:
+            self._update_security_parameter(sec_param="Minimum",
+                                            ignore=ignore)
+            acc_service.update_min_passwd_length(passwd_length)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard paramater '
+                          '``MinPasswordLength``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_ipmi_over_lan(self, enable=False, ignore=False):
+        """Update the IPMI/DCMI_Over_LAN security param.
+
+        :param enable: A boolean param, True when IPMI/DCMI_Over_LAN needs to
+               be enabled. If passed False, IPMI/DCMI_Over_LAN security param
+               will be disabled. If nothing passed default will be False.
+        :param ignore : A boolean param, True when IPMI/DCMI_Over_LAN needs to
+               be ignored. If passed False, IPMI/DCMI_Over_LAN security param
+               will not be ignored. If nothing passed default will be False.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            self._update_security_parameter(sec_param="IPMI", ignore=ignore)
+            sushy_manager.networkprotocol.update_ipmi_enabled(enable)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard paramater '
+                          '``IPMI/DCMI_Over_LAN``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_authentication_failure_logging(self, logging_threshold=None,
+                                              ignore=False):
+        """Update the Authentication_failure_Logging security param.
+
+        :param logging_threshold: Value of authenication failure logging
+               threshold. If nothing passed default will be None.
+        :param ignore : A boolean param, True when
+               Authentication_failure_Logging needs to be ignored. If passed
+               False, Authentication_failure_Logging security param will not
+               be ignored. If nothing passed default will be False.
+        :raises: IloError, on an error from iLO.
+        """
+        acc_service = self._sushy.get_account_service()
+        try:
+            self._update_security_parameter(sec_param="Failure Logging",
+                                            ignore=ignore)
+            acc_service.update_auth_failure_logging(logging_threshold)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard paramater '
+                          '``Authentication_failure_Logging``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def update_secure_boot(self, enable=True, ignore=False):
+        """Update Secure_Boot security param on the server.
+
+        :param enable: A boolean param, True when Secure_Boot needs to be
+               enabled. If passed False, Secure_Boot security param will
+               be disabled. If nothing passed default will be True.
+        :param ignore : A boolean param, True when Secure_boot needs to be
+               ignored. If passed False, Secure_boot security param will
+               not be ignored. If nothing passed default will be False.
+        :raises: IloError, on an error from iLO.
+        """
+        try:
+            self._update_security_parameter(sec_param="Secure Boot",
+                                            ignore=ignore)
+            self.set_secure_boot_mode(enable)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to update the '
+                          'security dashboard paramater ``Secure_boot``. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def _create_csr(self, csr_params):
+        """Creates the Certificate Signing Request.
+
+        :param csr_params: A dictionary containing all the necessary
+               information required to create CSR.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_man = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            cert_request = (
+                sushy_man.securityservice.https_certificate_uri.generate_csr(
+                    csr_params))
+
+            (fd, temp_file) = tempfile.mkstemp(suffix='.csr')
+
+            with open(temp_file, 'w') as f:
+                f.write(cert_request)
+
+            return temp_file
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to create the '
+                          'certificate signing request. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def add_ssl_certificate(self, csr_params, signed_cert,
+                            private_key, pass_phrase):
+        """Creates CSR and adds the signed SSL certificate to the iLO.
+
+        :param csr_params: A dictionary containing all the necessary
+               information required to create CSR.
+        :param signed_cert: signed certificate which will be used
+               to sign the created CSR.
+        :param private_key: private key.
+        :param pass_phrase: Pass phrase for the private key.
+        :raises: IloError, on an error from iLO.
+        """
+        csr_file = self._create_csr(csr_params)
+
+        (fd, temp_file) = tempfile.mkstemp(suffix='.ext')
+        (fd, https_cert_file) = tempfile.mkstemp(suffix='.crt')
+        (fd, ss_cert_file) = tempfile.mkstemp(suffix='.crt')
+
+        with open(signed_cert, 'r') as f:
+            data = json.dumps(f.read())
+        p = re.sub(r"\"", "", data)
+        q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+        c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+        if len(c_list) == 0:
+            msg = (self._("No valid certificate in %(cert_file)s.") %
+                   {"cert_file": signed_cert})
+            LOG.debug(msg)
+            raise exception.InvalidParameterValueError(msg)
+
+        ss_cert = c_list[0]
+
+        with open(ss_cert_file, 'w') as f:
+            f.write(ss_cert)
+
+        content = [
+            "authorityKeyIdentifier = keyid,issuer\n",
+            "basicConstraints = CA:true,pathlen:1\n",
+            "keyUsage = digitalSignature,keyEncipherment,keyCertSign,cRLSign",
+            "\nextendedKeyUsage = clientAuth,serverAuth\n",
+            "subjectKeyIdentifier = hash"]
+
+        with open(temp_file, 'w') as f:
+            f.writelines(content)
+
+        cert_cmd = (
+            "openssl x509 -req -days 365 -in %(csr_file)s -extfile"
+            " %(tempfile)s -CA %(cert)s -CAkey %(p_key)s -passin "
+            " pass:%(pphrase)s -CAcreateserial -out %(cert_file)s"
+            % {'csr_file': csr_file, 'tempfile': temp_file,
+               'cert': ss_cert_file, 'p_key': private_key,
+               'pphrase': pass_phrase, 'cert_file': https_cert_file})
+        try:
+            process = subprocess.Popen(cert_cmd, stdout=subprocess.PIPE,
+                                       stderr=subprocess.PIPE, shell=True)
+            out, err = process.communicate()
+        except Exception as e:
+            msg = (self._("Failed to create HTTPS certificate. "
+                          "error: %(err)s") % {"err": e})
+            LOG.debug(msg)
+            raise exception.CertificateCreationError(msg)
+
+        self._add_https_certificate(https_cert_file)
+
+    def _add_https_certificate(self, cert_file):
+        """Adds the signed https certificate to the iLO.
+
+        :param certificate: Signed HTTPS certificate file.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_man = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            with open(cert_file, 'r') as f:
+                data = json.dumps(f.read())
+            p = re.sub(r"\"", "", data)
+            q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+            c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+            if len(c_list) == 0:
+                msg = (self._("No valid certificate in %(cert_file)s.") %
+                       {"cert_file": cert_file})
+                LOG.debug(msg)
+                raise exception.IloError(msg)
+
+            cert = c_list[0]
+            sushy_man.securityservice.https_certificate_uri.import_certificate(
+                cert)
+            common.wait_for_ilo_after_reset(self)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to import the '
+                          'given certificate. '
+                          'Error: %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def create_csr(self, path, csr_params):
+        """Creates the Certificate Signing Request.
+
+        :param path: directory to store csr file.
+        :param csr_params: A dictionary containing all the necessary
+               information required to create CSR.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_man = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            cert_request = (
+                sushy_man.securityservice.https_certificate_uri.generate_csr(
+                    csr_params))
+
+            if not os.path.exists(path):
+                os.makedirs(path, 0o755)
+
+            csr_file_name = os.path.basename(path)
+            csr_file = os.path.join(path, csr_file_name + '.csr')
+
+            with open(csr_file, 'w') as f:
+                f.write(cert_request)
+            os.chmod(csr_file, 0o755)
+
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to create the '
+                          'certificate signing request. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def add_https_certificate(self, cert_file):
+        """Adds the signed https certificate to the iLO.
+
+        :param certificate: Signed HTTPS certificate file.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_man = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            with open(cert_file, 'r') as f:
+                data = json.dumps(f.read())
+            p = re.sub(r"\"", "", data)
+            q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+            c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+            if len(c_list) == 0:
+                msg = (self._("No valid certificate in %(cert_file)s.") %
+                       {"cert_file": cert_file})
+                LOG.debug(msg)
+                raise exception.IloError(msg)
+
+            cert = c_list[0]
+            sushy_man.securityservice.https_certificate_uri.import_certificate(
+                cert)
+            common.wait_for_ilo_after_reset(self)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to import the '
+                          'given certificate. '
+                          'Error: %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def get_security_dashboard_values(self):
         """Gets all the parameters related to security dashboard.
 
         :return: a dictionary of the security dashboard values
@@ -672,15 +1090,16 @@ class RedfishOperations(operations.IloOperations):
         sec_capabilities = {}
         sushy_manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
         try:
-
             security_dashboard = (
                 sushy_manager.securityservice.securitydashboard)
             security_params = (
-                sushy_manager.securityservice.securityparamscollectionuri)
+                security_dashboard.securityparamscollectionuri)
+            if security_dashboard.server_configuration_lock_status:
+                sec_capabilities.update(
+                    {'server_configuration_lock_status': (
+                     security_dashboard.server_configuration_lock_status)})
             sec_capabilities.update(
-                {'server_configuration_lock_status': (
-                 security_dashboard.server_configuration_lock_status),
-                 'overall_security_status': (
+                {'overall_security_status': (
                  security_dashboard.overall_status)})
             security_parameters = {}
             param_members = security_params.get_members()
@@ -712,7 +1131,7 @@ class RedfishOperations(operations.IloOperations):
         :returns: a dictionary of only those security parameters and their
             security status which are applicable for ironic.
         """
-        values = self._get_security_dashboard_values()
+        values = self.get_security_dashboard_values()
         ironic_sec_capabilities = {}
         ironic_sec_capabilities.update(
             {'overall_security_status': values.get('overall_security_status')})
@@ -732,9 +1151,9 @@ class RedfishOperations(operations.IloOperations):
         raises: IloError on an error from iLO.
         """
         capabilities = {}
-
         sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
         sushy_manager = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        sushy_chassis = self._get_sushy_chassis(PROLIANT_CHASSIS_ID)
         try:
             count = len(sushy_system.pci_devices.gpu_devices)
             boot_mode = rf_utils.get_supported_boot_mode(
@@ -798,6 +1217,12 @@ class RedfishOperations(operations.IloOperations):
                      'logical_nvdimm_n': (
                      json.dumps(memory_data.has_logical_nvdimm_n))})
 
+            gpu_capabilities = gpu_common.gpu_capabilities(sushy_system,
+                                                           sushy_chassis)
+            for member in gpu_capabilities:
+                for key in member:
+                    capabilities.update(member.get(key))
+
             capabilities.update(
                 self._parse_security_dashboard_values_for_capabilities())
 
@@ -948,8 +1373,8 @@ class RedfishOperations(operations.IloOperations):
             # local_gb = sushy_system.storage_summary
             prop = {'memory_mb': (sushy_system.memory_summary.size_gib * 1024),
                     'cpus': sushy_system.processors.summary.count,
-                    'cpu_arch': sushy_map.PROCESSOR_ARCH_VALUE_MAP_REV.get(
-                    sushy_system.processors.summary.architecture),
+                    'cpu_arch': CPU_ARCH_MAP.get(
+                        sushy_system.processors.summary.architecture),
                     'local_gb': common_storage.get_local_gb(sushy_system)}
             return {'properties': prop,
                     'macs': sushy_system.ethernet_interfaces.summary}
@@ -1470,10 +1895,55 @@ class RedfishOperations(operations.IloOperations):
             msg = 'TLS certificate cannot be upload in BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
 
-    def remove_tls_certificate(self, cert_file_list):
-        """Removes the TLS certificate from the iLO.
+    def _get_fps_from_file(self, cert_file):
+        """Gets the finger prints from the certificate file.
 
-        :param cert_file_list: List of TLS certificate files
+        Parse the file passed in to get the certificates. For each certificate,
+        find the fingerprint by calculating the digest of base64 decoded
+        content. Finally return the list of the fingerprints.
+
+        :param cert_file: TLS certificate file containing one or more
+               certificates.
+
+        :return: Returns the list of FPs for the certificates in the file.
+        """
+        fp_list = []
+        with open(cert_file, 'r') as f:
+            data = json.dumps(f.read())
+            p = re.sub(r"\"", "", data)
+            q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+            c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+            if len(c_list) == 0:
+                LOG.warning("Could not find any valid certificate in "
+                            "%(cert_file)s. Ignoring." %
+                            {"cert_file": cert_file})
+                return fp_list
+
+            for content in c_list:
+                pem_lines = [line.strip() for line in (
+                    content.strip().split('\n'))]
+
+                try:
+                    der_data = b64decode(''.join(pem_lines[1:-1]))
+                except ValueError:
+                    LOG.warning("Illegal base64 encountered "
+                                "in the certificate.")
+                else:
+                    cert = load_certificate(FILETYPE_ASN1, der_data)
+                    fp = cert.digest('sha256').decode('ascii')
+                    fp_list.append(fp)
+        return fp_list
+
+    def remove_tls_certificate(self, cert_file_list=[],
+                               excl_cert_file_list=[]):
+        """Removes the TLS CA certificates from the iLO.
+
+        :param cert_file_list: List of TLS CA certificate files
+        :param excl_cert_file_list: List of TLS CA certificate files to be
+               retained on the iLO. These certificates will not be
+               removed from the iLO.
 
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedError, if the command is
@@ -1481,56 +1951,61 @@ class RedfishOperations(operations.IloOperations):
         """
         sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
 
-        if(self._is_boot_mode_uefi()):
-            cert_dict = {}
-            del_cert_list = []
-            for cert_file in cert_file_list:
-                with open(cert_file, 'r') as f:
-                    data = json.dumps(f.read())
-                    p = re.sub(r"\"", "", data)
-                    q = re.sub(r"\\n", "\r\n", p).rstrip()
-
-                    c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
-
-                    if len(c_list) == 0:
-                        LOG.warning("Could not find any valid certificate in "
-                                    "%(cert_file)s. Ignoring." %
-                                    {"cert_file": cert_file})
-                        continue
-
-                    for content in c_list:
-                        pem_lines = [line.strip() for line in (
-                            content.strip().split('\n'))]
-
-                        try:
-                            der_data = b64decode(''.join(pem_lines[1:-1]))
-                        except ValueError:
-                            LOG.warning("Illegal base64 encountered "
-                                        "in the certificate.")
-                        else:
-                            cert = load_certificate(FILETYPE_ASN1, der_data)
-                            fp = cert.digest('sha256').decode('ascii')
-                            cert_fp = {
-                                "FingerPrint": fp
-                            }
-                            del_cert_list.append(cert_fp)
-
-            if len(del_cert_list) == 0:
-                msg = (self._("No valid certificate in %(cert_file_list)s.") %
-                       {"cert_file_list": cert_file_list})
-                raise exception.IloError(msg)
-
-            cert_dict.update({"DeleteCertificates": del_cert_list})
-
-            try:
-                (sushy_system.bios_settings.tls_config.
-                 tls_config_settings.remove_tls_certificate(cert_dict))
-            except sushy.exceptions.SushyError as e:
-                msg = (self._("The Redfish controller has failed to remove "
-                              "TLS certificate. Error %(error)s") %
-                       {'error': str(e)})
-                LOG.debug(msg)
-                raise exception.IloError(msg)
-        else:
-            msg = 'TLS certificate cannot be removed in BIOS boot mode'
+        if not self._is_boot_mode_uefi():
+            msg = 'TLS certificates cannot be removed in BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
+
+        cert_dict = {}
+        del_cert_list = []
+        exc_fp_list = []
+
+        for exc_cert_file in excl_cert_file_list:
+            efp_list = self._get_fps_from_file(exc_cert_file)
+            exc_fp_list.extend(efp_list)
+
+        LOG.debug("Excluding certificates with FingerPrints: %(exc_fp_list)s",
+                  {'exc_fp_list': exc_fp_list})
+
+        if not cert_file_list:
+            tls_certificates = (sushy_system.bios_settings.tls_config.
+                                tls_certificates)
+            for cert in tls_certificates:
+                fp = cert.get("FingerPrint")
+
+                if fp not in exc_fp_list:
+                    cert_fp = {
+                        "FingerPrint": fp
+                    }
+                    del_cert_list.append(cert_fp)
+
+        else:
+            all_fp_list = []
+
+            for cert_file in cert_file_list:
+                afp_list = self._get_fps_from_file(cert_file)
+                all_fp_list.extend(afp_list)
+
+            final_fp_set = set(all_fp_list) - set(exc_fp_list)
+
+            for fp in final_fp_set:
+                cert_fp = {
+                    "FingerPrint": fp
+                }
+                del_cert_list.append(cert_fp)
+
+        if len(del_cert_list) == 0:
+            msg = (self._("No valid certificate in %(cert_file_list)s.") %
+                   {"cert_file_list": cert_file_list})
+            raise exception.IloError(msg)
+
+        cert_dict.update({"DeleteCertificates": del_cert_list})
+
+        try:
+            (sushy_system.bios_settings.tls_config.
+             tls_config_settings.remove_tls_certificate(cert_dict))
+        except sushy.exceptions.SushyError as e:
+            msg = (self._("The Redfish controller has failed to remove "
+                          "TLS certificate. Error %(error)s") %
+                   {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
