@@ -1,4 +1,4 @@
-# Copyright 2017 Hewlett Packard Enterprise Development LP
+# Copyright 2017-2022 Hewlett Packard Enterprise Development LP
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -33,13 +33,18 @@ class PCIDevice(base.ResourceBase):
 
     sub_class_code = base.Field('SubclassCode')
 
+    odata_id = base.Field('@odata.id')
+
+    vendor_id = base.Field('VendorID')
+
     @property
     @sushy_utils.cache_it
     def nic_capacity(self):
-        for item in self.name.split():
-            if 'Gb' in item:
-                capacity = item.strip('Gb')
-                return int(capacity) if capacity.isdigit() else 0
+        if self.name:
+            for item in self.name.split():
+                if 'Gb' in item:
+                    capacity = item.strip('Gb')
+                    return int(capacity) if capacity.isdigit() else 0
         return 0
 
 
@@ -52,11 +57,12 @@ class PCIDeviceCollection(base.ResourceCollectionBase):
     @property
     @sushy_utils.cache_it
     def gpu_devices(self):
+        """Returns the list of gpu devices uris"""
         gpu_devices = []
         for member in self.get_members():
             if member.class_code in CLASSCODE_FOR_GPU_DEVICES:
                 if member.sub_class_code in SUBCLASSCODE_FOR_GPU_DEVICES:
-                    gpu_devices.append(member)
+                    gpu_devices.append(member.odata_id)
         return gpu_devices
 
     @property
@@ -64,3 +70,13 @@ class PCIDeviceCollection(base.ResourceCollectionBase):
     def max_nic_capacity(self):
         """Gets the maximum NIC capacity"""
         return str(max([m.nic_capacity for m in self.get_members()])) + 'Gb'
+
+    @property
+    @sushy_utils.cache_it
+    def vendor_id(self):
+        """Returns the vendor_id dictionary"""
+        vendor_id = {}
+        for member in self.get_members():
+            if member.odata_id in self.gpu_devices:
+                vendor_id.update({member.identity: member.vendor_id})
+        return vendor_id
