@@ -15,6 +15,7 @@
 __author__ = 'HPE'
 
 import re
+
 import sushy
 from sushy.resources import base
 from sushy.resources.system import system
@@ -38,7 +39,6 @@ from proliantutils.redfish.resources.system.storage import simple_storage
 from proliantutils.redfish.resources.system.storage import \
     smart_storage as hpe_smart_storage
 from proliantutils.redfish.resources.system.storage import storage
-
 from proliantutils.redfish import utils
 
 
@@ -48,7 +48,8 @@ PERSISTENT_BOOT_DEVICE_MAP = {
     'CDROM': sushy.BOOT_SOURCE_TARGET_CD,
     'NETWORK': sushy.BOOT_SOURCE_TARGET_PXE,
     'ISCSI': sushy.BOOT_SOURCE_TARGET_UEFI_TARGET,
-    'HDD': sushy.BOOT_SOURCE_TARGET_HDD
+    'HDD': sushy.BOOT_SOURCE_TARGET_HDD,
+    'UEFIHTTP': sushy.BOOT_SOURCE_TARGET_UEFI_HTTP
 }
 
 
@@ -59,10 +60,18 @@ class PowerButtonActionField(base.CompositeField):
     target_uri = base.Field('target', required=True)
 
 
+class OneButtonSecureEraseActionField(base.CompositeField):
+    target_uri = base.Field('target', required=True)
+
+
 class HpeActionsField(base.CompositeField):
     computer_system_ext_powerbutton = (
         PowerButtonActionField('#HpeComputerSystemExt.PowerButton'))
 
+    computer_system_ext_one_button_secure_erase = (
+        OneButtonSecureEraseActionField(
+            '#HpeComputerSystemExt.SecureSystemErase'))
+
 
 class HPESystem(system.System):
     """Class that extends the functionality of System resource class
@@ -104,6 +113,42 @@ class HPESystem(system.System):
 
         return push_action
 
+    def _get_hpe_one_button_secure_erase_action_element(self):
+        one_button_secure_erase_action = (
+            self._hpe_actions.computer_system_ext_one_button_secure_erase)
+        if not one_button_secure_erase_action:
+            raise exception.MissingAttributeError(
+                attribute=(
+                    'Oem/Hpe/Actions/#HpeComputerSystemExt.SecureSystemErase'),
+                resource=self.path)
+
+        return one_button_secure_erase_action
+
+    def do_one_button_secure_erase(self):
+        """Perform the one button secure erase on the hardware.
+
+        The One-button secure erase process resets iLO and deletes all licenses
+        stored there, resets BIOS settings, and deletes all AHS and warranty
+        data stored on the system. It also erases supported non-volatile
+        storage data and deletes any deployment settings profiles.
+
+        :raises: IloError, on an error from iLO.
+        """
+        try:
+            target_uri = (
+                self._get_hpe_one_button_secure_erase_action_element().
+                target_uri)
+            data = {
+                "SystemROMAndiLOErase": True,
+                "UserDataErase": True
+            }
+            self._conn.post(target_uri, data=data)
+        except sushy.exceptions.SushyError as e:
+            msg = ("The Redfish controller failed to perform one button "
+                   "secure erase operation on the hardware. Error: %(error)s"
+                   % {'error': str(e)})
+            raise exception.IloError(msg)
+
     def push_power_button(self, target_value):
         """Reset the system in hpe exclusive manner.
 
@@ -315,7 +360,7 @@ class HPESystem(system.System):
             try:
                 ssc_obj = self.get_smart_storage_config(config_id)
                 ssc_obj.delete_raid()
-            except exception.IloLogicalDriveNotFoundError as e:
+            except exception.IloLogicalDriveNotFoundError:
                 ld_exc_count += 1
             except sushy.exceptions.SushyError as e:
                 any_exceptions.append((config_id, str(e)))
@@ -534,7 +579,7 @@ class HPESystem(system.System):
                 if ssc_obj:
                     result = ssc_obj.read_raid(controller=controller)
                     config['logical_disks'].extend(result['logical_disks'])
-            except exception.IloLogicalDriveNotFoundError as e:
+            except exception.IloLogicalDriveNotFoundError:
                 ld_exc_count += 1
             except sushy.exceptions.SushyError as e:
                 any_exceptions.append((controller, str(e)))
